
module ROTConfig

  # shall we use weak hashmaps to store the compounds and roles
  WEAKREFERENCE=true

  # shall we autoconnect undefined callin methods
  AUTOCONNECT=true
  
  #  DEBUG => 5
  #  INFO  => 4
  #  WARN  => 3
  #  ERROR => 2
  #  FAIL  => 1
  #  NO    => 0
  LOGLEVEL = $DEBUG ? 4 : 0 #This means: if we are in ruby's debug mode, the loglevel is INFO otherwise NO

  # Display which method is called? (SLOW!!)
  LOGMETHODINFO = true

end

