require "objectteam"
require "date"
require "readline"
require "person"
require "constraint"
require "deploy"

check = DomainConstraintHealing.new
jack = Person.new("London", "Jack", Date.today)

check.while_active{
    jack.set_name("Herbert")
    #error: no special characters 
    jack.set_name("Klara:")
    #error: name less than 3 letter
    jack.set_first_name("Bl")
    #error: date not in range -> tomorrow
    jack.set_date_of_birth(Date.today.succ)

    puts jack.to_s

}
