
class Person
    attr_accessor :name
    attr_accessor :first_name
    attr_accessor :date_of_birth

    def initialize(name, first_name, date_of_birth)
        @name = name
        @first_name = first_name
        @date_of_birth = date_of_birth
    end

    def set_name(name)
        @name = name
    end

    def set_first_name(first_name)
        @first_name = first_name
    end

    def set_date_of_birth(date_of_birth)
        @date_of_birth = date_of_birth
    end
    
    def to_s
        "Name: #{@name}\nFirst Name: #{@first_name}\nBirthday: #{date_of_birth}\n"
    end
end
