
class Graphic
  def paint
  end
end

class Figure < Graphic
  def paint()
    print "===Figure drawed\n"
  end
end

class Group < Graphic
  def paint
    print "===Group drawed\n"
  end
end

class Line < Graphic
  def paint()
    print "Line drawed\n"
  end
end

class Square < Graphic
  def paint()
    print "Square drawed\n"
  end
end

class Text < Graphic
  def paint()
    print "Text drawed\n"
  end
end
