require "objectteam"
require "person"
require "factory"

bakery = Bakery.new
steelwork = SteelWork.new

chef = Chef.new

puts "\nNormal behaviour==========================================\n"
chef.delegate_work

puts "\nWork in Bakery ===========================================\n"
bakery.while_active{
  chef.delegate_work
}

puts "\nWork in Steelwork ========================================\n"
steelwork.while_active{
  chef.delegate_work
}
